#include <iostream>
#include <fstream>
#include <strstream>
#include <complex>
#include <vector>
#include <stack>
#include <list>
#include <string>
#include <cassert>

using namespace std;

#include "cmplex.h"

cmplex ii(0.0, 1.0);

cmplex::cmplex(const double& x, const double& y) {
      this->x = x;
      this->y = y;
}

cmplex::cmplex(const double& x) {
      this->x = x;
      this->y = 0.0;
}

cmplex::cmplex(const cmplex& a) {
      this->x = a.x;
      this->y = a.y;
}

cmplex::cmplex() {
      this->x = 0.0;
      this->y = 0.0;
}

void cmplex::print() const {
    cout << " (" << x << ") + i (" << y << ")" << endl;
}

void cmplex::print(ofstream& fout) const {
    fout << " (" << x << ") + i (" << y << ")";
}

void cmplex::setby(const double& x, const double& y) {
    this->x = x;
    this->y = y;
}

void cmplex::write(ofstream& fout) const {
    fout.write((char*) &x, sizeof x);
    fout.write((char*) &y, sizeof y);
}

void cmplex::read(ifstream& fin) {
    fin.read((char*) &x, sizeof x);
    fin.read((char*) &y, sizeof y);
}

void cmplex::operator=(const cmplex& a) {
    x = a.x;
    y = a.y;
}

void cmplex::operator=(const double& ar) {
    x = ar;
    y = 0.0;
}

cmplex cmplex::operator-() const {
    double x = -this->x;
    double y = -this->y;
    return cmplex(x, y);
}

void cmplex::operator+=(const cmplex& a) {
    x += a.x;
    y += a.y;
}

void cmplex::operator+=(const double& ar) {
    x += ar;
}

void cmplex::operator-=(const cmplex& a) {
    x -= a.x;
    y -= a.y;
}

void cmplex::operator-=(const double& ar) {
    x -= ar;
}

cmplex cmplex::operator+(const cmplex& a) const {
    return cmplex(x + a.x, y + a.y);
}

cmplex cmplex::operator+(const double& ar) const {
    return cmplex(x + ar, y);
}

cmplex cmplex::operator-(const cmplex& a) const {
    return cmplex(x - a.x, y - a.y);
}

cmplex cmplex::operator-(const double& ar) const {
    return cmplex(x - ar, y);
}

cmplex cmplex::operator*(const cmplex& a) const {
    return cmplex(x * a.x - y * a.y, x * a.y + y * a.x);
}

cmplex cmplex::operator*(const double& a) const {
    return cmplex(x * a, y * a);
}

cmplex cmplex::operator/(const cmplex& a) const {
    return cmplex(
          (x * a.x - y * a.y) / (a.x*a.x+a.y*a.y),
          (y * a.x - x * a.y) / (a.x*a.x+a.y*a.y));
}

cmplex cmplex::operator/(const double& val) const {
    return cmplex(x / val, y / val);
}


bool cmplex::operator==(const cmplex& a) const {
    return (x == a.x && y == a.y);
}

bool cmplex::operator!=(const cmplex& a) const {
    return !(x == a.x && y == a.y);
}


//------------------------

cmplex operator*(const double& val, const cmplex& a) {
  return cmplex(val * a.x , val * a.y);
}

cmplex operator/(const double& val, const cmplex& a) {
    return cmplex(
           (val * a.x) / (a.x*a.x+a.y*a.y),
          -(val * a.y) / (a.x*a.x+a.y*a.y));
}

cmplex Prod(const cmplex& a, const cmplex& b) {
    return cmplex(a.x * b.x - a.y * b.y,
                  a.x * b.y + a.y * b.x);
}

cmplex Conj(const cmplex& a) {
    return cmplex(a.x, -a.y);
}

double Re(const cmplex& a) {
    return a.x;
}

double Im(const cmplex& a) {
    return a.y;
}
