#include <iostream>
#include <fstream>
#include <strstream>
#include <complex>
#include <vector>
#include <stack>
#include <list>
#include <string>
#include <cassert>

using namespace std;

#include "vectIII.h"
#include "qsimplex.h"
#include "line.h"

line::line(const vectZ1& Lsize,
           const vectR1& Lmax,
           const vectR1& Lmin) {
    setby(Lsize, Lmax, Lmin);
}

void line::setby(const vectZ1& Lsize,
                      const vectR1& Lmax,
                      const vectR1& Lmin) {
    Lsize_ = Lsize;
    Lmax_  = Lmax;
    Lmin_  = Lmin;
    dB_    = (Lmax_ -  Lmin_) / Lsize;
    cout << "Lsize_" << Lsize_ << endl;
    SpZ_.setby(  Lsize_, 
                 Lmax_ - dB_ / 2.0, 
                 Lmin_ + dB_ / 2.0, 
                 dB_);
    SpI_.setby(Lsize_, 
               Lmax_ - vectR1(dB_ / 2.0), 
               Lmin_ + vectR1(dB_ / 2.0), 
               dB_);
}

void line::print() const {
    cout << "line : ";
    cout << "(" << Lsize_ << "), " << endl;
    cout << "(" << Lmax_ << "), " << endl;
    cout << "(" << Lmin_ << "), " << endl;
    cout << "(" << dB_ << "), " << endl;
    SpZ_.print();
    SpI_.print();
    cout << endl;
}

void line::print(ofstream& fout) const {
    fout << "line : ";
    fout << "(" << Lsize_ << "), " << endl;
    fout << "(" << Lmax_ << "), " << endl;
    fout << "(" << Lmin_ << "), " << endl;
    fout << "(" << dB_ << "), " << endl;
    SpZ_.print(fout);
    SpI_.print(fout);
    fout << endl; 
}

void line::write(ofstream& fout) const {
    fout.write((char*) &Lsize_, sizeof Lsize_);
    fout.write((char*) &Lmax_,  sizeof Lmax_);
    fout.write((char*) &Lmin_,  sizeof Lmin_);
    fout.write((char*) &dB_,    sizeof dB_);
    SpZ_.write(fout);
    SpI_.write(fout);
}

void line::read(ifstream& fin) {
    fin.read((char*) &Lsize_, sizeof Lsize_);
    fin.read((char*) &Lmax_,  sizeof Lmax_);
    fin.read((char*) &Lmin_,  sizeof Lmin_);
    fin.read((char*) &dB_,    sizeof dB_);
    SpZ_.read(fin);
    SpI_.read(fin);
}

// I -> I (gradient(), convertSV())
iSqI line::Ip(const iSqI& iI) const {
    return SpI_.iserial(SpI_.ipos(iI) + (1));
}

iSqI line::Im(const iSqI& iI) const{
    return SpI_.iserial(SpI_.ipos(iI) + (-1));
}


// I -> Z (for sheaf<Z,V> convert)
iSqZ line::Zp(const iSqI& iI) const {
    return SpZ_.iserial(SpI_.ipos(iI) + (1));
}

iSqZ line::Zm(const iSqI& iI) const {
    return SpZ_.iserial(SpI_.ipos(iI) + (0));
}

//Zero --> I
iSqI line::Ip(const iSqZ& iZ) const {
    return SpI_.iserial(SpZ_.ipos(iZ) + (0));
}

iSqI line::Im(const iSqZ& iZ) const {
    return SpI_.iserial(SpZ_.ipos(iZ) + (-1));
}


// Zero -> Zero
iSqZ line::Zp(const iSqZ& iZ) const { 
    return SpZ_.iserial(SpZ_.ipos(iZ) + (1));
}

iSqZ line::Zm(const iSqZ& iZ) const {
    return SpZ_.iserial(SpZ_.ipos(iZ) + (-1));
}

