#include <iostream>
#include <fstream>
#include <strstream>
#include <complex>
#include <vector>
#include <stack>
#include <list>
#include <string>
#include <cassert>

using namespace std;

#include "cmplex.h"
#include "vectII.h"
#include "matC2x2.h"

matC2x2::matC2x2(const cmplex& a11, const cmplex& a12,
                 const cmplex& a21, const cmplex& a22) {
      this->a11 = a11;
      this->a12 = a12;
      this->a21 = a21;
      this->a22 = a22;
}

matC2x2::matC2x2(const double& a11, const double& a12,
                 const double& a21, const double& a22) {
      this->a11 = a11;
      this->a12 = a12;
      this->a21 = a21;
      this->a22 = a22;
}

matC2x2::matC2x2(const matC2x2& m) {
      this->a11 = m.a11;
      this->a12 = m.a12;
      this->a21 = m.a21;
      this->a22 = m.a22;
}

matC2x2::matC2x2() {
      this->a11 = 0;
      this->a12 = 0;
      this->a21 = 0;
      this->a22 = 0;
}

void matC2x2::print() const {
      a11.print(); cout << ", ";
      a12.print(); cout << endl;
      a21.print(); cout << ", ";
      a22.print(); cout << endl;
}

void matC2x2::print(ofstream& fout) const {
      a11.print(fout); fout << ", ";
      a12.print(fout); fout << endl;
      a21.print(fout); fout << ", ";
      a22.print(fout); fout << endl;
}

void matC2x2::setby(const cmplex& a11, const cmplex& a12,
                    const cmplex& a21, const cmplex& a22) {
      this->a11 = a11;
      this->a12 = a12;
      this->a21 = a21;
      this->a22 = a22;
}

void matC2x2::setby(const double& a11, const double& a12,
                    const double& a21, const double& a22) {
      this->a11 = a11;
      this->a12 = a12;
      this->a21 = a21;
      this->a22 = a22;
}

void matC2x2::write(ofstream& fout) const {
    a11.write(fout);
    a12.write(fout);
    a21.write(fout);
    a22.write(fout);
}

void matC2x2::read(ifstream& fin) {
    a11.read(fin);
    a12.read(fin);
    a21.read(fin);
    a22.read(fin);
}

void matC2x2::operator=(const matC2x2& m) {
    this->a11 = m.a11;
    this->a12 = m.a12;
    this->a21 = m.a21;
    this->a22 = m.a22;
}

matC2x2 matC2x2::operator-() const {
    cmplex a11 = -this->a11;
    cmplex a12 = -this->a12;
    cmplex a21 = -this->a21;
    cmplex a22 = -this->a22;
    return matC2x2(a11, a12, a21, a22);
}

void matC2x2::operator+=(const matC2x2& m) {
    this->a11 += m.a11;
    this->a12 += m.a12;
    this->a21 += m.a21;
    this->a22 += m.a22;
}

void matC2x2::operator-=(const matC2x2& m) {
    this->a11 -= m.a11;
    this->a12 -= m.a12;
    this->a21 -= m.a21;
    this->a22 -= m.a22;
}

matC2x2 matC2x2::operator+(const matC2x2& m) const {
    return matC2x2(a11 + m.a11, a12 + m.a12,
                   a21 + m.a21, a22 + m.a22);
}

matC2x2 matC2x2::operator-(const matC2x2& m) const {
    return matC2x2(a11 - m.a11, a12 - m.a12,
                   a21 - m.a21, a22 - m.a22);
}

matC2x2 matC2x2::operator*(const matC2x2& m) const {
    cmplex b11 = a11 * m.a11 + a12 * m.a21;
    cmplex b12 = a11 * m.a12 + a12 * m.a22;
    cmplex b21 = a21 * m.a11 + a22 * m.a21;
    cmplex b22 = a21 * m.a12 + a22 * m.a22;
    return matC2x2(b11, b12, b21, b22);
}

vectC2 matC2x2::operator*(const vectC2& v) const {
    cmplex x = a11 * v.x + a12 * v.y;
    cmplex y = a21 * v.x + a22 * v.y;
    return vectC2(x, y);
}

matC2x2 matC2x2::operator*(const double& a) const {
    return matC2x2(a11 * a, a12 * a,
                   a21 * a, a22 * a);
}

matC2x2 matC2x2::operator*(const cmplex& a) const {
    return matC2x2(a11 * a, a12 * a,
                   a21 * a, a22 * a);
}

matC2x2 matC2x2::operator/(const double& a) const {
    return matC2x2(a11 / a, a12 / a,
                   a21 / a, a22 / a);
}

matC2x2 matC2x2::operator/(const cmplex& a) const {
    return matC2x2(a11 / a, a12 / a,
                   a21 / a, a22 / a);
}

cmplex det(const matC2x2& m) {
    return m.a11 * m.a22 - m.a12 * m.a21;
}

matC2x2 inv(const matC2x2& m) {
    cmplex invdet = 1.0 / det(m); 
    return matC2x2(m.a22 * invdet, -m.a12 * invdet,
                  -m.a21 * invdet,  m.a11 * invdet);
}

matC2x2 tranps(const matC2x2& m) {
    return matC2x2(m.a11, m.a21, m.a12, m.a22);
}

matC2x2 Hconj(const matC2x2& m) {
    return matC2x2(Conj(m.a11), Conj(m.a21), Conj(m.a12), Conj(m.a22));
}

bool matC2x2::operator==(const matC2x2& m) const {
    return (a11 == m.a11 && a12 == m.a12 
         && a21 == m.a21 && a22 == m.a22);
}

bool matC2x2::operator!=(const matC2x2& m) const {
    return !(a11 == m.a11 && a12 == m.a12 
          && a21 == m.a21 && a22 == m.a22);
}


//------------------------
matC2x2 operator*(const cmplex& val, const matC2x2& m) {
  return matC2x2(val * m.a11 , val * m.a12,
                 val * m.a21 , val * m.a22);
}

matC2x2 operator*(const double& val, const matC2x2& m) {
  return matC2x2(val * m.a11 , val * m.a12,
                 val * m.a21 , val * m.a22);
}

matC2x2 operator/(const double& val, const matC2x2& m) {
    return val * inv(m);
}

matC2x2 operator/(const cmplex& val, const matC2x2& m) {
    return val * inv(m);
}

matC2x2 Prod(const matC2x2& m1, const matC2x2& m2) {
    cmplex b11 = m1.a11 * m2.a11 + m1.a12 * m2.a21;
    cmplex b12 = m1.a11 * m2.a12 + m1.a12 * m2.a22;
    cmplex b21 = m1.a21 * m2.a11 + m1.a22 * m2.a21;
    cmplex b22 = m1.a21 * m2.a12 + m1.a22 * m2.a22;
    return matC2x2(b11, b12, b21, b22);
}

