#ifndef morphism_included
#define morphism_included

class oboxZ {
    vectZ1 max_;
    vectZ1 min_;
    bool    inner_;
    char    BCtype_;
public:
    oboxZ(const vectZ1& maxi, const vectZ1& mini) : 
          max_(maxi), min_(mini), inner_(true), BCtype_('P') {}
    oboxZ(const vectZ1& maxi, const vectZ1& mini, bool inner, char bctype) {
        setby(maxi, mini, inner, bctype);
    }
    void setby(const vectZ1& maxi, const vectZ1& mini) {
        max_    = maxi;
        min_    = mini;
        inner_  = true;
        BCtype_ = 'P';
    }
    void setby(const vectZ1& maxi, const vectZ1& mini, 
                bool inner, char bctype) {
        max_    = maxi;
        min_    = mini;
        inner_  = inner;
        BCtype_ = bctype;
    }
    oboxZ()  {}
    ~oboxZ() {}
    const vectZ1& omax() const { return max_;    }
    const vectZ1& omin() const { return min_;    }
    const bool&   inner() const { return inner_;  }
    const char&  BCtype() const { return BCtype_; }

    bool ok(const vectZ1& ipnt) const{
        ///return true;
        if (BCtype_ == 'P') return true;
        if (inner_) {
            return (max_ >= ipnt) && (min_ <= ipnt);
        } else {
            return !((max_ > ipnt) && (min_ < ipnt));
        }
    }

    void read(ifstream& fin){
        fin.read((char*) &max_,    sizeof max_);
        fin.read((char*) &min_,    sizeof min_);
        fin.read((char*) &inner_,  sizeof inner_);
        fin.read((char*) &BCtype_, sizeof BCtype_);
    }
    void write(ofstream& fout){
        fout.write((char*) &max_,    sizeof max_);
        fout.write((char*) &min_,    sizeof min_);
        fout.write((char*) &inner_,  sizeof inner_);
        fout.write((char*) &BCtype_, sizeof BCtype_);
    }
};

class morphism {
protected:
    const oboxZ*  Regq_;
public:
    morphism(const oboxZ& Reg) : Regq_(&Reg) {}
    morphism() {}
    ~morphism() {}
    void tuned(const oboxZ& Reg) {
        Regq_ = &Reg;
    }
    const oboxZ& Reg() const { return *Regq_; }
};

class conjugate : public morphism {
public:
   sheaf<QsmpI, cmplex> operator()(const sheaf<QsmpI, cmplex>& f);
};

class gradient : public morphism {
public:
    sheaf<QsmpZ, double> operator()(const sheaf<QsmpI, double>& f);
};

class divergence : public morphism {
public:
   sheaf<QsmpI, double> operator()(const sheaf<QsmpZ, double>&  f); 
};

class convert : public morphism {
public:
    convert() {}
    ~convert() {}
    sheaf<QsmpZ, int>    operator()(const sheaf<QsmpI, int>& f);
    sheaf<QsmpZ, double> operator()(const sheaf<QsmpI, double>& f);
};

class pairing : public morphism {
public:
    pairing() {}
    ~pairing() {}
    double  operator()(const sheaf<QsmpI, double>& f,
                       const sheaf<QsmpI, double>& g);
    double  operator()(const sheaf<QsmpZ, double>& f,
                       const sheaf<QsmpZ, double>& g);
};

class maxabsS : public morphism {
public:
    maxabsS() {}
    ~maxabsS() {}
    double  operator()(const sheaf<QsmpI, double>& f);
    double  operator()(const sheaf<QsmpI, cmplex>& f);
};

class multiply : public morphism {
public:
    multiply() {}
    ~multiply() {}
    sheaf<QsmpI, vectC2> operator()(const sheaf<QsmpI, matC2x2>& m,
                                    const sheaf<QsmpI, vectC2>&  u);
    sheaf<QsmpZ, vectC2> operator()(const sheaf<QsmpZ, matC2x2>& m,
                                    const sheaf<QsmpZ, vectC2>&  u);
};

class translate : public morphism {
public:
    translate() {}
    ~translate() {}
    sheaf<QsmpI, vectC2> operator()(const sheaf<QsmpZ, matC2x2>& m,
                                    const sheaf<QsmpI, vectC2>&  u);
    sheaf<QsmpZ, vectC2> operator()(const sheaf<QsmpZ, matC2x2>& m,
                                    const sheaf<QsmpZ, vectC2>&  u);
};

class absolute : public morphism {
public:
    absolute() {}
    ~absolute() {}
    sheaf<QsmpI, double> operator()(const sheaf<QsmpI, vectC2>& u);
    sheaf<QsmpZ, double> operator()(const sheaf<QsmpZ, vectC2>& u);
};

class envelope : public morphism {
public:
    envelope() {}
    ~envelope() {}
//    sheaf<QsmpI, double> operator()(const sheaf<QsmpI, vectC2>& u);
    sheaf<QsmpZ, double> operator()(const sheaf<QsmpZ, double>& u);
};


extern gradient   Grad;
extern divergence Div;
extern pairing    Pairing;
extern maxabsS    MaxabsS;
extern convert    Convert;
extern translate  Transl;
extern multiply   Multi;
extern absolute   Abs;
extern envelope    Envlp;

void   tuned(const oboxZ& obox);
void   tunedConvert(const oboxZ& obox, oboxZ& obconv);

#endif
