#include <iostream>
#include <fstream>
#include <strstream>
#include <complex>
#include <vector>
#include <stack>
#include <list>
#include <string>
#include <cassert>

using namespace std;

#include "cmplex.h"
#include "vectII.h"
#include "vectIII.h"
#include "qsimplex.h"

//----------------------------------------------------------------------------
//  qsimplex Qsmp 
//----------------------------------------------------------------------------
qsimplex::qsimplex(const vectZ1& Lsize,
                   const vectR1& Lmax,
                   const vectR1& Lmin,
                   const vectR1& dB) {
    setby(Lsize, Lmax, Lmin, dB);
}

void qsimplex::setby(const vectZ1& Lsize,
                     const vectR1& Lmax,
                     const vectR1& Lmin,
                     const vectR1& dB) {
    Lsize_   = Lsize;
    Lmax_    = Lmax;
    Lmin_    = Lmin;
    dB_      = dB;
    nXs_     = Lsize_;
    npoints_ = Lsize_;
}

vectZ1 qsimplex::ipos(const isequence& ipnt) const {
     int ix =  ipnt() % nXs_;
     return vectZ1(ix);
}

vectR1 qsimplex::dpos(const isequence& ipnt) const {
     int    ix =  ipnt() % nXs_;
     double dx = ix * dB_ + Lmin_;
     return vectR1(dx);
}

isequence qsimplex::iserial(const vectZ1& ivct) const {
    vectZ1 jvct = ivct + Lsize_;
    return isequence((jvct % Lsize_));
}

//isequence qsimplex::iserial(int ix) const {
//    int jx = ix + Lsize_;
//    return isequence((jx % Lsize_));
//}

void qsimplex::print() const {
    cout << Lsize_ << ", ";
    cout << Lmax_ << ", ";
    cout << Lmin_ << ", ";
    cout << dB_ << ", ";
    cout << " " << nXs_  << ", ";
    cout << " " << npoints_ << ", " << endl;
}

void qsimplex::print(ofstream& fout) const {
    fout << Lsize_ << ", ";
    fout << Lmax_ << ", ";
    fout << Lmin_ << ", ";
    fout << dB_ << ", ";
    fout << " " << nXs_  << ", ";
    fout << " " << npoints_ << ", " << endl;
}

void qsimplex::write(ofstream& fout) const {
    fout.write((char*) &Lsize_,   sizeof Lsize_);
    fout.write((char*) &Lmax_,    sizeof Lmax_);
    fout.write((char*) &Lmin_,    sizeof Lmin_);
    fout.write((char*) &dB_,      sizeof dB_);
    fout.write((char*) &nXs_,     sizeof nXs_);
    fout.write((char*) &npoints_, sizeof npoints_);
}

void qsimplex::read(ifstream& fin) {
    fin.read((char*) &Lsize_,   sizeof Lsize_);
    fin.read((char*) &Lmax_,    sizeof Lmax_);
    fin.read((char*) &Lmin_,    sizeof Lmin_);
    fin.read((char*) &dB_,      sizeof dB_);
    fin.read((char*) &nXs_,     sizeof nXs_);
    fin.read((char*) &npoints_, sizeof npoints_);
}


void qsimplexZero::print() const {
    cout << "qsimplexZero " << endl;
    qsimplex::print();
}

void qsimplexZero::print(ofstream& fout) const {
    fout << "qsimplexZero " << endl;
    qsimplex::print(fout);
}

vectZ1 qsimplexZero::ipos(const isequenceZero& ipnt) const {
    return qsimplex::ipos(ipnt);
}

vectR1 qsimplexZero::dpos(const isequenceZero& ipnt) const {
    return qsimplex::dpos(ipnt);
}

isequenceZero qsimplexZero::iserial(const vectZ1& ivct) const {
    return isequenceZero(qsimplex::iserial(ivct)());
}

//isequenceZero qsimplexZero::iserial(int ix) const {
//     return isequenceZero(qsimplex::iserial(ix)());
//}

//----------------------------------------------------------------------------
//  qsimplexI QsmpI 
//----------------------------------------------------------------------------
void QsmpI::print() const {
    cout << "qsimplexI " << endl;
    qsimplex::print();
}

void QsmpI::print(ofstream& fout) const {
    fout << "QsmplexI " << endl;
    qsimplex::print(fout);
}

vectZ1 QsmpI::ipos(const iSqI& ipnt) const {
    int ix =  ipnt() % nXs_;
    return vectZ1(ix);
}

vectR1 QsmpI::dpos(const iSqI& ipnt) const {
    int    ix =  ipnt() % nXs_;
    double dx = ix * dB_ + Lmin_;
    return vectR1(dx);
}

iSqI QsmpI::iserial(const vectZ1& ivct) const {
    return iSqI(qsimplex::iserial(ivct)());
}

//iSqI QsmpI::iserial(int ix) const {
//     return iSqI(qsimplex::iserial(ix)());
//}

