#ifndef qsimplex_included
#define qsimplex_included

class isequence {
   public: 
    int       i;
    isequence(){ }
    isequence(const int& j) : i(j) {}
    isequence(const isequence& iS) : i(iS.i) {}
    ~isequence(){ }
    const int& operator()() const { return i; }
          int& operator()()       { return i; }
    void operator=(const int& j) { i = j; }
    void operator=(const isequence& iS) { i = iS.i; }
    void operator++() { i++; }  
    void operator--() { i--; }  
    //isequence operator++() { /// NG! 
    //    isequence temp = *this;
    //    i++;
    //    return temp;
    //}
    isequence operator++(int notused) { 
        isequence temp = *this;
        i++;
        return temp;
    }
    isequence operator--(int notused) { 
        isequence temp = *this;
        i--;
        return temp;
    }
    bool operator!=(const isequence& iS) const { return (i != iS.i); }
    bool operator==(const isequence& iS) const { return (i == iS.i); }
    bool operator<=(const isequence& iS) const { return (i <= iS.i); }
    bool operator>=(const isequence& iS) const { return (i >= iS.i); }
    bool operator<( const isequence& iS)  const { return (i < iS.i);  }
    bool operator>( const isequence& iS)  const { return (i > iS.i);  }
};

class isequenceZero : public isequence {
   public:
    isequenceZero(){ }
    isequenceZero(const isequenceZero& iS) : isequence(iS) {}
    isequenceZero(const int& j) : isequence(j) {}
};
class isequenceI   : public isequence {
   public:
    isequenceI(){ }
    isequenceI(const isequenceI& iS) : isequence(iS) {}
    isequenceI(const int& j) : isequence(j) {}
};

class qsimplex {
 protected:
    vectZ1    Lsize_;
    vectR1    Lmin_;
    vectR1    Lmax_;
    vectR1    dB_;
    int       npoints_;
    int       nXs_;

 public:
    qsimplex() {}
    ~qsimplex(){}
    qsimplex(const vectZ1& Lsize,
            const vectR1& Lmax,
            const vectR1& Lmin,
            const vectR1& dB);
    void  setby(const vectZ1& Lsize, 
                const vectR1& Lmax,
                const vectR1& Lmin,
                const vectR1& dB);
    vectZ1   ipos(const isequence& ipnt)     const;
    vectR1   dpos(const isequence& ipnt)     const;
    isequence iserial(const vectZ1& ivct)    const;
//    isequence iserial(int ix) const;
    void print() const;
    void print(ofstream& fout) const;
    void read(ifstream& fin);
    void write(ofstream& fout) const;

    const vectZ1& Lsize()   const { return Lsize_;   }
    const vectR1& dL()      const { return dB_;      }
    const vectR1& dB()      const { return dB_;      }
    const vectR1& Lmin()    const { return Lmin_;    }
    const vectR1& Lmax()    const { return Lmax_;    }
    const int      npoints() const { return npoints_; }
    isequence      begin()   const { return 0;        }
    isequence      end()     const { return npoints_; }
};

class qsimplexZero : public qsimplex {
 public:
    qsimplexZero() : qsimplex() {}
    ~qsimplexZero(){}
    void print() const;
    void print(ofstream& fout) const;
    vectZ1   ipos(const isequenceZero& ipnt)     const;
    vectR1   dpos(const isequenceZero& ipnt)     const;
    isequenceZero iserial(const vectZ1& ivct)    const;
//    isequenceZero iserial(int ix) const;
    isequenceZero begin()   const { return 0;        }
    isequenceZero end()     const { return npoints_; }
};

class qsimplexI : public qsimplex {
 public:
    qsimplexI() : qsimplex() {}
    ~qsimplexI(){}
    void print() const;
    void print(ofstream& fout) const;
    vectZ1   ipos(const isequenceI& ipnt)     const;
    vectR1   dpos(const isequenceI& ipnt)     const;
    isequenceI iserial(const vectZ1& ivct)    const;
//    isequenceI iserial(int ix) const;
    isequenceI begin()   const { return 0;        }
    isequenceI end()     const { return npoints_; }
};


typedef isequenceZero  iSqZ;
typedef isequenceI     iSqI;
typedef qsimplexZero   QsmpZ;
typedef qsimplexI      QsmpI;

#endif
