#include <iostream>
#include <fstream>
#include <strstream>
#include <complex>
#include <vector>
#include <stack>
#include <list>
#include <string>
#include <cassert>

using namespace std;

#include "cmplex.h"
#include "vectII.h"
#include "vectIII.h"
#include "matC2x2.h"
#include "qsimplex.h"
#include "line.h"
#include "sheaf.h"

template<>
  void sheaf<QsmpZ, double>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpZ();
      ndata_ = Spq_->npoints();
      oq_ = new double[ ndata_ ];
      valinit();
  }

template<>
  void sheaf<QsmpZ, int>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpZ();
      ndata_ = Spq_->npoints();
      oq_ = new int[ ndata_ ];
      valinit();
  }

template<>
  void sheaf<QsmpZ, cmplex>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpZ();
      ndata_ = Spq_->npoints();
      oq_ = new cmplex[ ndata_ ];
      valinit();
  }

template<>
  void sheaf<QsmpZ, vectR3>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpZ();
      ndata_ = Spq_->npoints();
      oq_ = new vectR3[ ndata_ ];
      valinit();
  }

template<>
  void sheaf<QsmpZ, vectR2>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpZ();
      ndata_ = Spq_->npoints();
      oq_ = new vectR2[ ndata_ ];
      valinit();
  }

template<>
  void sheaf<QsmpZ, vectC2>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpZ();
      ndata_ = Spq_->npoints();
      oq_ = new vectC2[ ndata_ ];
      valinit();
  }


template<>
  void sheaf<QsmpZ, matC2x2>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpZ();
      ndata_ = Spq_->npoints();
      oq_ = new matC2x2[ ndata_ ];
      valinit();
  }

/////////////////////////////////

template<>
  void sheaf<QsmpI, double>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpI();
      ndata_ = Spq_->npoints();
      oq_ = new double[ ndata_ ];
      valinit();
  }

template<>
  void sheaf<QsmpI, int>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpI();
      ndata_ = Spq_->npoints();
      oq_ = new int[ ndata_ ];
      valinit();
  }

template<>
  void sheaf<QsmpI, cmplex>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpI();
      ndata_ = Spq_->npoints();
      oq_ = new cmplex[ ndata_ ];
      valinit();
  }

template<>
  void sheaf<QsmpI, vectR3>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpI();
      ndata_ = Spq_->npoints();
      oq_ = new vectR3[ ndata_ ];
      valinit();
  }

template<>
  void sheaf<QsmpI, vectC2>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpI();
      ndata_ = Spq_->npoints();
      oq_ = new vectC2[ ndata_ ];
      valinit();
  }

template<>
  void sheaf<QsmpI, vectR2>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpI();
      ndata_ = Spq_->npoints();
      oq_ = new vectR2[ ndata_ ];
      valinit();
  }

template<>
  void sheaf<QsmpI, matC2x2>::setby(const line& L) {
      Lq_    = &L;
      Spq_   = &L.SpI();
      ndata_ = Spq_->npoints();
      oq_ = new matC2x2[ ndata_ ];
      valinit();
  }




template<>
vectR3 sheaf<QsmpZ, vectR3>::operator()(const vectR1& pnt) const{
    const sheaf<QsmpZ, vectR3>& This = *this;

    double dix = (pnt - Spq_->Lmin()) / Spq_->dL();
    int ix = (int) dix;

    double perx = dix - ix;
    vectR3 rtnv
    = (1.0 - perx) * This(ix) + perx * This(ix + 1);
    return rtnv;
}

template<>
double sheaf<QsmpZ, double>::operator()(const vectR1& pnt) const{
    const sheaf<QsmpZ, double>& This = *this;

    double dix = (pnt - Spq_->Lmin()) / Spq_->dL();
    int ix = (int) dix;

    double perx = dix - ix;
    double rtnd = (1.0 - perx) * This(ix) + perx * This(ix + 1);
    return rtnd;
}

template<>
cmplex sheaf<QsmpZ, cmplex>::operator()(const vectR1& pnt) const{
    const sheaf<QsmpZ, cmplex>& This = *this;

    double dix = (pnt - Spq_->Lmin()) / Spq_->dL();
    int ix = (int) dix;

    double perx = dix - ix;
    cmplex rtnd = (1.0 - perx) * This(ix) + perx * This(ix + 1);
    return rtnd;
}


template<> 
  sheaf<QsmpZ, vectR3>::sheaf(const line& L) {
      setby(L);
  }

template<> 
  sheaf<QsmpZ, vectR3>::sheaf(const sheaf<QsmpZ, vectR3>& f){
      setby(f);
  }

template<> 
  sheaf<QsmpZ, cmplex>::sheaf(const sheaf<QsmpZ, cmplex>& f){
      setby(f);
  }

template<> 
  vectR3& sheaf<QsmpZ, vectR3>::operator()(const isequence& i) {
      return *(oq_ + i() );
  }

//////////////////////////////////////////////////////////////////////////////

template<class Qsmplex, class Value> 
  void sheaf<Qsmplex, Value>::valinit(){
      for (int id = 0; id < ndata_; id ++) {
          ///*(oq_ + id) = 0;
          eachvalinit((oq_ + id));
      }
  }

template<class Qsmplex, class Value> 
  sheaf<Qsmplex, Value>::sheaf(const line& L) {
      setby(L);
  }

template<class Qsmplex, class Value> 
  sheaf<Qsmplex, Value>::sheaf(const sheaf<Qsmplex, Value>& f){
      setby(f);
  }

template<class Qsmplex, class Value> 
  void sheaf<Qsmplex, Value>::setby(const sheaf<Qsmplex, Value>& f){
      Lq_    = f.Lq_;    ///add
      Spq_   = f.Spq_;
      ndata_ = f.ndata_;
      oq_    = f.oq_;
      oq_    = new Value[ ndata_ ];
      for (int id = 0; id < ndata_; id ++) {
          *(oq_ + id) = *(f.oq_ + id);
      }
  }

template<class Qsmplex, class Value> 
  Value& sheaf<Qsmplex, Value>::operator()(const isequence& i) {
      return *(oq_ + i() );
  }

template<class Qsmplex, class Value> 
  const Value& sheaf<Qsmplex, Value>::operator()(const isequence& i) const {
      return *(oq_ + i() );
  }

template<class Qsmplex, class Value> 
  Value& sheaf<Qsmplex, Value>::operator()(const int& ix) { 
     return *(oq_ + Spq_->iserial(ix)());
  }

template<class Qsmplex, class Value> 
  const Value& sheaf<Qsmplex, Value>::operator()(const int& ix) const {
      return *(oq_ + Spq_->iserial(ix)());
  }


///*just for compiler
//template<class Qsmplex, class Value> 
//  Value  sheaf<Qsmplex, Value>::operator()(const vectR1& pnt) const {
//     return static_cast<Value>(0);
//  }

template<class Qsmplex, class Value> 
  void sheaf<Qsmplex, Value>::operator=(const sheaf<Qsmplex, Value>& f) {
      for( int i = 0; i< ndata_ ; i++ ){
          *( oq_ + i ) = *( f.oq_ + i );
      }
  }

template<class Qsmplex, class Value> 
  void sheaf<Qsmplex, Value>::operator+=(const sheaf<Qsmplex, Value>& f) {
      for( int i = 0; i< ndata_ ; i++ ){
          *( oq_ + i ) += *( f.oq_ + i );
      }
  }

///void operator-=(const sheaf<Qsmplex, Value>& f);
template<class Qsmplex, class Value> 
  void sheaf<Qsmplex, Value>::operator-=(const sheaf<Qsmplex, Value>& f) {
      for( int i = 0; i< ndata_ ; i++ ){
          *( oq_ + i ) -= *( f.oq_ + i );
      }
  }

 ///void operator+=(const Value& c);
template<class Qsmplex, class Value> 
  void sheaf<Qsmplex, Value>::operator+=(const Value& c) {
      for( int i = 0; i< ndata_ ; i++ ){
          *( oq_ + i ) += c;
      }
  }

///void operator-=(const Value& c);
template<class Qsmplex, class Value> 
  void sheaf<Qsmplex, Value>::operator-=(const Value& c) {
      for( int i = 0; i< ndata_ ; i++ ){
          *( oq_ + i ) -= c;
      }
  }

///add 08/29 ??
template<class Qsmplex, class Value> 
  sheaf<Qsmplex, Value> sheaf<Qsmplex, Value>::operator-() {
      for( int i = 0; i< ndata_ ; i++ ){
          *( oq_ + i ) = -(*( oq_ + i ));
      }
      return *this;
  }

///sheaf<Qsmplex, Value> operator*(const sheaf<Qsmplex, Value>& f);
/*
template<class Qsmplex, class Value> 
  sheaf<Qsmplex, Value> sheaf<Qsmplex, Value>::operator*(
                  const sheaf<Qsmplex, Value>& f) {
      sheaf<Qsmplex, Value> rtnsf(f.L());
      for(isequence iq = f.begin(); iq != f.end(); ++iq ) {
          rtnsf(iq) = this->operator()(iq) * f(iq);
      }
      return rtnsf;
  }
*/

template<class Qsmplex, class Value> 
  sheaf<Qsmplex, Value> sheaf<Qsmplex, Value>::multiS(
                  const sheaf<Qsmplex, Value>& f) {
      sheaf<Qsmplex, Value> rtnsf(f.L());
      for(isequence iq = f.begin(); iq != f.end(); ++iq ) {
          rtnsf(iq) = this->operator()(iq) * f(iq);
      }
      return rtnsf;
  }

///sheaf<Qsmplex, Value> operator+(const sheaf<Qsmplex, Value>& f);
template<class Qsmplex, class Value> 
  sheaf<Qsmplex, Value> sheaf<Qsmplex, Value>::operator+(
                  const sheaf<Qsmplex, Value>& f) {
      sheaf<Qsmplex, Value> rtnsf(f.L());
      for(isequence iq = f.begin(); iq != f.end(); ++iq ) {
         rtnsf(iq) = this->operator()(iq) + f(iq);
      }
      return rtnsf;
  }

///sheaf<Qsmplex, Value> operator-(const sheaf<Qsmplex, Value>& f);
template<class Qsmplex, class Value> 
  sheaf<Qsmplex, Value> sheaf<Qsmplex, Value>::operator-(
                  const sheaf<Qsmplex, Value>& f) {
      sheaf<Qsmplex, Value> rtnsf(f.L());
      for(isequence iq = f.begin(); iq != f.end(); ++iq ) {
         rtnsf(iq) = this->operator()(iq) - f(iq);
      }
      return rtnsf;
  }

template<class Qsmplex, class Value> 
  void sheaf<Qsmplex, Value>::write(ofstream& fout) const {
      fout.write((char*) &ndata_, sizeof ndata_);
      for( int id  = 0; id < ndata_ ; id ++ ){
          Value r = *(oq_ + id);
          fout.write((char*) &r, sizeof r);
      }
  }

template<class Qsmplex, class Value> 
  bool sheaf<Qsmplex, Value>::read(ifstream& fin) {
      int ndata;
      fin.read((char*) &ndata, sizeof ndata);
      if (ndata_ != ndata) {
          cerr << "error in sheaf::read()" << endl; 
          return false;
      }
      for( int id  = 0; id < ndata_ ; id ++ ){
          Value r;
          fin.read((char*) &r, sizeof r);
          *(oq_ + id) = r;
      }
      return true;
  }

///friend define
sheaf<QsmpI, double> operator*(const double& a,
                               const sheaf<QsmpI, double>& b) {
    sheaf<QsmpI, double> rtnsf(b.L());
    for(isequence iq = b.begin(); iq != b.end(); ++iq ) {
        rtnsf(iq) = a * b(iq);
    }
    return rtnsf;
}

sheaf<QsmpZ, double> operator*(const double& a,
                                 const sheaf<QsmpZ, double>& b) {
    sheaf<QsmpZ, double> rtnsf(b.L());
    for(isequence iq = b.begin(); iq != b.end(); ++iq ) {
        rtnsf(iq) = a * b(iq);
    }
    return rtnsf;
}

sheaf<QsmpI, cmplex> operator*(const cmplex& a,
                               const sheaf<QsmpI, cmplex>& b) {
    sheaf<QsmpI, cmplex> rtnsf(b.L());
    for(isequence iq = b.begin(); iq != b.end(); ++iq ) {
        rtnsf(iq) = a * b(iq);
    }
    return rtnsf;
}

sheaf<QsmpZ, cmplex> operator*(const cmplex& a,
                                 const sheaf<QsmpZ, cmplex>& b) {
    sheaf<QsmpZ, cmplex> rtnsf(b.L());
    for(isequence iq = b.begin(); iq != b.end(); ++iq ) {
        rtnsf(iq) = a * b(iq);
    }
    return rtnsf;
}

sheaf<QsmpI, cmplex> operator*(const double& a,
                                 const sheaf<QsmpI, cmplex>& b) {
    sheaf<QsmpI, cmplex> rtnsf(b.L());
    for(isequence iq = b.begin(); iq != b.end(); ++iq ) {
        rtnsf(iq) = a * b(iq);
    }
    return rtnsf;
}

sheaf<QsmpZ, cmplex> operator*(const double& a,
                                 const sheaf<QsmpZ, cmplex>& b) {
    sheaf<QsmpZ, cmplex> rtnsf(b.L());
    for(isequence iq = b.begin(); iq != b.end(); ++iq ) {
        rtnsf(iq) = a * b(iq);
    }
    return rtnsf;
}


sheaf<QsmpI, double> operator*(const sheaf<QsmpI, double>& f,
                                 const sheaf<QsmpI, double>& g) {
    const line& L = f.L();
    sheaf<QsmpI, double> rtnsf(L);
    for(iSqI isq = L.SpI().begin(); isq != L.SpI().end(); ++isq ) {
        rtnsf(isq) = f(isq) * g(isq);
    }
    return rtnsf;
}

sheaf<QsmpZ, double> operator*(const sheaf<QsmpZ, double>& f,
                                 const sheaf<QsmpZ, double>& g) {
    const line& L = f.L();
    sheaf<QsmpZ, double> rtnsf(L);
    for(iSqI isq = L.SpI().begin(); isq != L.SpI().end(); ++isq ) {
        rtnsf(isq) = f(isq) * g(isq);
    }
    return rtnsf;
}


sheaf<QsmpI, cmplex> operator*(const sheaf<QsmpI, cmplex>& f,
                                 const sheaf<QsmpI, cmplex>& g) {
    const line& L = f.L();
    sheaf<QsmpI, cmplex> rtnsf(L);
    for(iSqI isq = L.SpI().begin(); isq != L.SpI().end(); ++isq ) {
        rtnsf(isq) = f(isq) * g(isq);
    }
    return rtnsf;
}

sheaf<QsmpZ, cmplex> operator*(const sheaf<QsmpZ, cmplex>& f,
                                 const sheaf<QsmpZ, cmplex>& g) {
    const line& L = f.L();
    sheaf<QsmpZ, cmplex> rtnsf(L);
    for(iSqI isq = L.SpI().begin(); isq != L.SpI().end(); ++isq ) {
        rtnsf(isq) = f(isq) * g(isq);
    }
    return rtnsf;
}


template class sheaf<QsmpZ, int>;
template class sheaf<QsmpZ, double>;
template class sheaf<QsmpZ, cmplex>;
template class sheaf<QsmpZ, vectR2>;
template class sheaf<QsmpZ, vectC2>;
template class sheaf<QsmpZ, vectR3>;
template class sheaf<QsmpZ, matC2x2>;
template class sheaf<QsmpI, int>;
template class sheaf<QsmpI, double>;
template class sheaf<QsmpI, cmplex>;
template class sheaf<QsmpI, vectR2>;
template class sheaf<QsmpI, vectC2>;
template class sheaf<QsmpI, vectR3>;
template class sheaf<QsmpI, matC2x2>;

