#ifndef sheaf_included
#define sheaf_included

template<class Qsimplex, class Value> class sheaf {
    Value*            oq_;
    const line*       Lq_;
    const Qsimplex*   Spq_;
    int               ndata_;

public:
    sheaf(const line& L);
    sheaf(const sheaf<Qsimplex, Value>& f);
    sheaf(){}
    ~sheaf() {
        SAFE_DELETE_ARRAY(oq_);
     }
    const line& L() const { return *Lq_; }
    void setby(const line& L);
    const Qsimplex&   Spq() const { return *Spq_; }
    void setby(const sheaf<Qsimplex, Value>& f);

          Value& operator()(const isequence& i);
    const Value& operator()(const isequence& i) const;
          Value& operator()(const vectZ1& ivec);
    const Value& operator()(const vectZ1& ivec) const;
          Value  operator()(const vectR1&  pnt) const; //for only SpZ-type
    const isequence begin() const { return Spq_->begin(); }
    const isequence end()   const { return Spq_->end(); };
    void operator= (const sheaf<Qsimplex, Value>& f);
    void operator+=(const sheaf<Qsimplex, Value>& f);
    void operator-=(const sheaf<Qsimplex, Value>& f);
    void operator-=(const Value& c);
    void operator+=(const Value& c);

    sheaf<Qsimplex, Value> operator-();
//    sheaf<Qsimplex, Value> operator*(const sheaf<Qsimplex, Value>& f);
    sheaf<Qsimplex, Value> multiS(const sheaf<Qsimplex, Value>& f);
    sheaf<Qsimplex, Value> operator+(const sheaf<Qsimplex, Value>& f);
    sheaf<Qsimplex, Value> operator-(const sheaf<Qsimplex, Value>& f);
    void write(ofstream& fout) const;
    bool read( ifstream& fin);
/*
    friend sheaf<Qsimplex, Value> operator*(const Value& a, 
                                           const sheaf<Qsimplex, Value>& b);
*/
    friend sheaf<Qsimplex, double> operator*(const double& a, 
                                            const sheaf<Qsimplex, double>& b);
    friend sheaf<Qsimplex, double> operator*(const sheaf<Qsimplex, double>& f,
                                            const sheaf<Qsimplex, double>& g);
    friend sheaf<Qsimplex, cmplex> operator*(const cmplex& a, 
                                             const sheaf<Qsimplex, cmplex>& b);
    friend sheaf<Qsimplex, cmplex> operator*(const double& a, 
                                             const sheaf<Qsimplex, cmplex>& b);
    friend sheaf<Qsimplex, cmplex> operator*(const sheaf<Qsimplex, cmplex>& f,
                                             const sheaf<Qsimplex, cmplex>& g);

private:
    void valinit();
    void eachvalinit(int*     iq) { *iq = 0; }
    void eachvalinit(double*  dq) { *dq = 0.0; }
    void eachvalinit(vectR2*  vq) { *vq = vectR2(0.0, 0.0); }
    void eachvalinit(vectC2*  vq) { *vq = vectC2(cmplex(0.0, 0.0), 
                                                 cmplex(0.0, 0.0)); }
    void eachvalinit(vectR3*  vq) { *vq = vectR3(0.0, 0.0, 0.0); }
    void eachvalinit(cmplex*  zq) { *zq = cmplex(0.0, 0.0); }
    void eachvalinit(matC2x2* mq) { *mq = matC2x2(0.0, 0.0, 0.0, 0.0); }
};

#endif
