#include <iostream>
#include <fstream>
#include <strstream>
#include <complex>
#include <vector>
#include <stack>
#include <list>
#include <string>
#include <cassert>

using namespace std;

#include "cmplex.h"
#include "vectII.h"

template<class T> 
  vectII<T>::vectII(const T& x, const T& y) {
        this->x = x;
        this->y = y;
  }

template<class T> 
  vectII<T>::vectII(const vectII<T>& a) {
        this->x = a.x;
        this->y = a.y;
  }

template<class T> 
  vectII<T>::vectII() {
        this->x = 0;
        this->y = 0;
  }

template<> 
  void vectII<double>::print() const {
      cout << " (" << x << ", " << y << ")" << endl;
  }

template<> 
  void vectII<int>::print() const {
      cout << " (" << x << ", " << y << ")" << endl;
  }

template<> 
  void vectII<cmplex>::print() const {
      cout << " (" << x.x << " + i " << x.y  
                   << y.x << " + i " << y.y <<  ")" << endl;
  }

template<>
  void vectII<int>::print(ofstream& fout) const {
      fout << " (" << x << ", " << y << ") ";
  }

template<>
  void vectII<double>::print(ofstream& fout) const {
      fout << " (" << x << ", " << y << ") ";
  }

template<>
  void vectII<cmplex>::print(ofstream& fout) const {
      fout << " (";
      x.print(fout);
      y.print(fout);
      fout << ")" << endl;
  }

template<class T>
  void vectII<T>::setby(const T& x, const T& y) {
      this->x = x;
      this->y = y;
  }

template<> 
  void vectII<int>::write(ofstream& fout) const {
      fout.write((char*) &x, sizeof x);
      fout.write((char*) &y, sizeof y);
  }

template<> 
  void vectII<double>::write(ofstream& fout) const {
      fout.write((char*) &x, sizeof x);
      fout.write((char*) &y, sizeof y);
  }

template<> 
  void vectII<cmplex>::write(ofstream& fout) const {
      x.write(fout);
      y.write(fout);
  }


template<> 
  void vectII<int>::read(ifstream& fin) {
      fin.read((char*) &x, sizeof x);
      fin.read((char*) &y, sizeof y);
  }

template<> 
  void vectII<double>::read(ifstream& fin) {
      fin.read((char*) &x, sizeof x);
      fin.read((char*) &y, sizeof y);
  }

template<> 
  void vectII<cmplex>::read(ifstream& fin) {
      x.read(fin);
      y.read(fin);
  }



template<class T> 
  void vectII<T>::operator=(const vectII<T>& a) {
      x = a.x;
      y = a.y;
  }

template<class T> 
  vectII<T> vectII<T>::operator-() const {
      T x = -this->x;
      T y = -this->y;
      return vectII<T>(x, y);
  }

template<class T> 
  void vectII<T>::operator+=(const vectII<T>& a) {
      x += a.x;
      y += a.y;
  }

template<class T> 
  void vectII<T>::operator-=(const vectII<T>& a) {
      x -= a.x;
      y -= a.y;
  }

template<class T> 
  vectII<T> vectII<T>::operator+(const T& val) const {
      return vectII<T>(x + val, y + val);
  }

template<class T> 
  vectII<T> vectII<T>::operator-(const T& val) const {
      return vectII<T>(x - val, y - val);
  }

template<class T> 
  vectII<T> vectII<T>::operator*(const T& val) const {
      return vectII<T>(x * val, y * val);
  }

template<class T> 
  vectII<T> vectII<T>::operator/(const T& val) const {
      return vectII<T>(x / val, y / val);
  }

template<class T> 
  vectII<T> vectII<T>::operator+(const vectII<T>& a) const {
      return vectII<T>(x + a.x, y + a.y);
  }

template<class T> 
  vectII<T> vectII<T>::operator-(const vectII<T>& a) const {
      return vectII<T>(x - a.x, y - a.y);
  }

// for sheaf
template<class T> 
  vectII<T> vectII<T>::operator*(const vectII<T>& a) const {
      T ycomp(0.0);
      return vectII<T>(x * a.y - y * a.x, ycomp);
  }

template<class T> 
  bool vectII<T>::operator==(const vectII<T>& a) const {
      return (x == a.x && y == a.y);
  }

vectR2 operator*(const double& val, const vectR2& a) {
    return vectR2(val * a.x , val * a.y);
}

double Eprod(const vectR2& a, const vectR2& b) {
    return a.x * b.y - a.y * b.x;
}

double Iprod(const vectR2& a, const vectR2& b) {
    return a.x * b.x + a.y * b.y;
}

vectC2 operator*(const cmplex& val, const vectC2& a) {
    return vectC2(val * a.x , val * a.y);
}

cmplex Hprod(const vectC2& a, const vectC2& b) {
    return Conj(a.x) * b.x + Conj(a.y) * b.y;
}

template class vectII<cmplex>;
template class vectII<double>;
template class vectII<int>;
