#include <iostream>
#include <fstream>
#include <strstream>
#include <complex>
#include <vector>
#include <stack>
#include <list>
#include <string>
#include <cassert>

using namespace std;

#include "vectIII.h"

template<class T> 
  vectIII<T>::vectIII(const T& x, const T& y, const T& z) {
        this->x = x;
        this->y = y;
        this->z = z;
  }

template<class T> 
  vectIII<T>::vectIII(const vectIII<T>& a) {
        this->x = a.x;
        this->y = a.y;
        this->z = a.z;
  }

template<class T> 
  vectIII<T>::vectIII() {
        this->x = 0;
        this->y = 0;
        this->z = 0;
  }

template<class T> 
  void vectIII<T>::print() const {
      cout << " (" << x << ", " << y << "," << z << ")" << endl;
  }

template<class T>
  void vectIII<T>::print(ofstream& fout) const {
      fout << " (" << x << ", " << y << "," << z << ") ";
  }

template<class T>
  void vectIII<T>::setby(const T& x, const T& y, const T& z) {
      this->x = x;
      this->y = y;
      this->z = z;
  }

template<class T> 
  void vectIII<T>::write(ofstream& fout) const {
      fout.write((char*) &x, sizeof x);
      fout.write((char*) &y, sizeof y);
      fout.write((char*) &z, sizeof z);
  }

template<class T> 
  void vectIII<T>::read(ifstream& fin) {
      fin.read((char*) &x, sizeof x);
      fin.read((char*) &y, sizeof y);
      fin.read((char*) &z, sizeof z);
  }

template<class T> 
  void vectIII<T>::operator=(const vectIII<T>& a) {
      x = a.x;
      y = a.y;
      z = a.z;
  }

template<class T> 
  vectIII<T> vectIII<T>::operator-() const {
      T x = -this->x;
      T y = -this->y;
      T z = -this->z;
      return vectIII<T>(x, y, z);
  }

template<class T> 
  void vectIII<T>::operator+=(const vectIII<T>& a) {
      x += a.x;
      y += a.y;
      z += a.z;
  }

template<class T> 
  void vectIII<T>::operator-=(const vectIII<T>& a) {
      x -= a.x;
      y -= a.y;
      z -= a.z;
  }

template<class T> 
  vectIII<T> vectIII<T>::operator+(const T& val) const {
      return vectIII<T>(x + val, y + val, z + val);
  }

template<class T> 
  vectIII<T> vectIII<T>::operator-(const T& val) const {
      return vectIII<T>(x - val, y - val, z - val);
  }

template<class T> 
  vectIII<T> vectIII<T>::operator*(const T& val) const {
      return vectIII<T>(x * val, y * val, z * val);
  }

template<class T> 
  vectIII<T> vectIII<T>::operator/(const T& val) const {
      return vectIII<T>(x / val, y / val, z / val);
  }

template<class T> 
  vectIII<T> vectIII<T>::operator+(const vectIII<T>& a) const {
      return vectIII<T>(x + a.x, y + a.y, z + a.z);
  }

template<class T> 
  vectIII<T> vectIII<T>::operator-(const vectIII<T>& a) const {
      return vectIII<T>(x - a.x, y - a.y, z - a.z);
  }

template<class T> 
  vectIII<T> vectIII<T>::operator*(const vectIII<T>& a) const {
      return vectIII<T>(y * a.z - z * a.y,
                        z * a.x - x * a.z,
                        x * a.y - y * a.x);
  }

template<class T> 
  bool vectIII<T>::operator==(const vectIII<T>& a) const {
      return (x == a.x && y == a.y && z == a.z);
  }

template<class T> 
  bool vectIII<T>::operator<=(const vectIII<T>& a) const {
      return (x <= a.x && y <= a.y && z <= a.z);
  }

template<class T> 
  bool vectIII<T>::operator>=(const vectIII<T>& a) const {
      return (x >= a.x && y >= a.y && z >= a.z);
  }

template<class T> 
  bool vectIII<T>::operator<(const vectIII<T>& a) const {
      return (x < a.x || y < a.y || z < a.z);
  }

template<class T> 
  bool vectIII<T>::operator>(const vectIII<T>& a) const {
      return (x > a.x || y > a.y || z > a.z);
  }

vectR3 operator*(const double& val, const vectR3& a) {
    return vectR3(val * a.x , val * a.y, val * a.z);
}

vectR3 Eprod(const vectR3& a, const vectR3& b) {
    return vectR3(a.y * b.z - a.z * b.y,
                   a.z * b.x - a.x * b.z,
                   a.x * b.y - a.y * b.x);
}

double Iprod(const vectR3& a, const vectR3& b) {
    return a.x * b.x + a.y * b.y + a.z * b.z;
}

template class vectIII<double>;
template class vectIII<int>;
