#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <fstream>
#include <strstream>
#include <complex>
#include <vector>
#include <stack>
#include <list>
#include <string>
#include <cassert>

using namespace std;

#include "cmplex.h"
#include "vectII.h"
#include "matC2x2.h"
#include "vectIII.h"
#include "qsimplex.h"
#include "line.h"
#include "sheaf.h"
#include "morphism.h"



int main(void) {

    vectZ1 Lsize =  6000;
    vectR1 Lmax  =  1.0;
    vectR1 Lmin  = -1.0;
    int    ntimes = 3000;


    line L(Lsize, Lmax, Lmin);
    sheaf<QsmpZ, matC2x2> M(L);
    sheaf<QsmpZ, vectC2>  u(L); 
    sheaf<QsmpZ, vectC2>  upre(L); 
    sheaf<QsmpZ, vectC2>  upost(L); 
    sheaf<QsmpZ, double>  uabs(L); 
    vector<sheaf<QsmpZ, double> >  ave_u2; 
    vector<sheaf<QsmpZ, double> >  env_ave_u2; 
    vector<double>                 qcog; 
    cmplex a11( 1.0/sqrt(2.0), 0.0);
    cmplex a12( 1.0/sqrt(2.0), 0.0);
    cmplex a21( 1.0/sqrt(2.0), 0.0);
    cmplex a22(-1.0/sqrt(2.0), 0.0);
    double omega = 0.0; // 0.7
    cmplex eomega(    cos(omega), sin(omega));
    cmplex inv_eomega(cos(omega),-sin(omega));
    //cmplex b12 = a12;
    //cmplex b11 = a11 * eomega;
    //cmplex b21 = a21;
    //cmplex b22 = a22 * inv_eomega;
    cmplex b11 = a11;
    cmplex b12 = a12 * eomega;
    cmplex b21 = a21 * inv_eomega;
    cmplex b22 = a22;


    matC2x2 Ma(a11, a12, a21, a22);
    //matC2x2 Mbm(b11, b12, b21, b22);
    //matC2x2 Mbp(a11, b12, a21, b22);

    int mtimes = 30; 
    for (int itime = 0; itime <= mtimes; itime ++) {
        sheaf<QsmpZ, double> ave(L); 
        for (iSqZ isq  = L.SpZ().begin();
                  isq != L.SpZ().end();   isq ++ ) {
            ave(isq) = 0.0; 
        }
        ave_u2.push_back(ave); 
        qcog.push_back(0.0);
    }
    FILE *fp;
    fp = fopen("file1.csv", "a");
    int nrands = 100;
    //int nrands = 20;
    for (int irand = 0; irand < nrands; irand ++) {
        cout << irand << endl;
        srand(irand + 2);
        vector<int> ipos;
        vector<int> jpos;

        int npnts = 300;
        for (int ipnt = 0; ipnt <npnts; ipnt ++) {
            int ip = rand() % Lsize;
            ipos.push_back(ip);
            jpos.push_back(0);
        }
        for (int jpnt = 0; jpnt <npnts; jpnt ++) {
            int jposmin = Lsize;
            int kpnt    = 0;
            for (int ipnt = 0; ipnt < npnts; ipnt ++) {
                 if (jposmin > ipos[ipnt]) {
	               jposmin = ipos[ipnt];
	               kpnt    = ipnt;
                 }
            }
	    jpos[jpnt] = ipos[kpnt];
	    ipos[kpnt] = Lsize + 1;
        }
        for (int jpnt = 0; jpnt < npnts - 1; jpnt ++) {
            if (jpos[jpnt] == jpos[jpnt+1]) {
                jpos[jpnt+1] = jpos[jpnt] + 1;
            }
            if (jpos[jpnt] >= jpos[jpnt+1]) {
                jpos[jpnt+1] = jpos[jpnt] + 1;
            }
        }
        int ix = 0;
        int iy = 0;
        int jpnt = 0;
        for (iSqZ isq  = L.SpZ().begin();
                  isq != L.SpZ().end();   isq ++ ) {
            M(isq) = Ma;
            //if (L.SpZ().ipos(isq) == Lsize/2  )  M(isq) = Mbm;
            if (L.SpZ().ipos(isq) == jpos[jpnt])  {
                double omega = (double)rand() * 0.8 / (double)RAND_MAX - 0.4;
                cmplex eomega(    cos(omega), sin(omega));
                cmplex inv_eomega(cos(omega),-sin(omega));
                if (rand() % 2 == 0) {
                     cmplex b11 = a11;
                     cmplex b12 = a12 * eomega;
                     cmplex b21 = a21 * inv_eomega;
                     cmplex b22 = a22;
                     matC2x2 Mbm(b11, b12, b21, b22);
                     M(isq) = Mbm;
                } else {
                     cmplex b11 = a11 * eomega;
                     cmplex b12 = a12;
                     cmplex b21 = a21;
                     cmplex b22 = a22 * inv_eomega;
                     matC2x2 Mbm(b11, b12, b21, b22);
                     M(isq) = Mbm;
                }
                jpnt ++;
            }
        }

        for (iSqZ isq  = L.SpZ().begin();
                  isq != L.SpZ().end();   isq ++ ) {
            if (L.SpZ().ipos(isq) == Lsize/2)  {
                cmplex u1 = cmplex(1.0/sqrt(2.0), 0.0          );
                cmplex u2 = cmplex(0.0,           1.0/sqrt(2.0));
                //cmplex u1 = cmplex(0.0, 0.0);
                //cmplex u2 = cmplex(0.0, 1.0);
                u(isq) = vectC2(u1, u2);
            } else {
                cmplex u1 = cmplex(0.0, 0.0);
                cmplex u2 = cmplex(0.0, 0.0);
                u(isq) = vectC2(u1, u2);
            }
            vectC2 uu = u(isq);
            //cout << uu.x.x << ", " << uu.x.y << ", ";
            //cout << uu.y.x << ", " << uu.y.y << endl;
        }
        int ditime = ntimes / mtimes;
        int jitime = 0;
        int jtime = 0;
        for (int itime = 0; itime < ntimes; itime ++) {
            upre  = u;
            upost = Transl(M, upre);
            u     = upost;
            uabs  = Abs(upost);
            //cout << itime << " : ";
            double sum = 0.0;
            for (iSqZ isq  = L.SpZ().begin();
                      isq != L.SpZ().end();   isq ++ ) {
                 sum += uabs(isq) * uabs(isq);
            }
            if (itime == jitime) {
                double qcogi = 0.0;
                double qwght = 0.0;
                for (iSqZ isq  = L.SpZ().begin();
                          isq != L.SpZ().end();   isq ++ ) {
                     //fprintf(fp, "%e, ", uabs(isq));
                     ave_u2[jtime](isq) += uabs(isq) * uabs(isq); 
                     int ipos = L.SpZ().ipos(isq);
                     if (ipos <= Lsize/2) {
                          qcogi += ipos * uabs(isq) * uabs(isq);
                          qwght += uabs(isq) * uabs(isq);
                     } 
                }
                qcog[jtime] += qcogi / qwght;
                //fprintf(fp, "\n");
                jtime ++;
                jitime += ditime;
	        if (jitime == ntimes) jitime--;
            }
            //cout << sum << endl;
        }
    }
    fprintf(fp, "\n");
    for (int jtime = 0; jtime <= mtimes; jtime ++) {
        for (iSqZ isq  = L.SpZ().begin();
                  isq != L.SpZ().end();   isq ++ ) {
            ave_u2[jtime](isq) /= nrands; 
            fprintf(fp, "%e, ", ave_u2[jtime](isq));
        }
        qcog[jtime] /= nrands;
        fprintf(fp, "\n");
    }
    fprintf(fp, "\n");
    fprintf(fp, "\n");
    for (int jtime = 0; jtime < mtimes; jtime ++) {
        sheaf<QsmpZ, double>  env_ave_u2I(L); 
        env_ave_u2I = Envlp(ave_u2[jtime]); 
        env_ave_u2.push_back(env_ave_u2I);
        for (iSqZ isq  = L.SpZ().begin();
                  isq != L.SpZ().end();   isq ++ ) {
            fprintf(fp, "%e, ", env_ave_u2I(isq)); 
        }
        fprintf(fp, "\n");
    }
    fprintf(fp, "\n");
    fprintf(fp, "\n");
    for (int jtime = 1; jtime < mtimes; jtime ++) {
        for (iSqZ isq  = L.SpZ().begin();
                  isq != L.SpZ().end();   isq ++ ) {
            fprintf(fp, "%e, ", 
               env_ave_u2[jtime](isq) - env_ave_u2[jtime - 1](isq)); 
        }
        fprintf(fp, "\n");
    }
//
    for (int jtime = 0; jtime < mtimes; jtime ++) {
        double  max = env_ave_u2[jtime](L.SpZ().begin());
        iSqZ    ismax;
        double  avei  = 0.0;
        double  avei2 = 0.0;
        int i = 0;
        for (iSqZ isq = L.SpZ().begin(); isq != L.SpZ().end(); ++isq) {
            if (max < env_ave_u2[jtime](isq)) {
                max = env_ave_u2[jtime](isq);
                ismax = isq;
            }
            avei  += i     * ave_u2[jtime](isq);
            avei2 += i * i * ave_u2[jtime](isq);
            i ++;
        }
        //avei  /= i;
        //avei2 /= i;
        int imax = L.SpZ().ipos(ismax);
        double  weight = 0.0;
        double iweight = 0.0;
        double  weightI = 0.0;
        double iweightI = 0.0;
        i = 0;
        for (iSqZ isq = L.SpZ().begin(); isq != L.SpZ().end(); isq ++) {
            if (L.SpZ().ipos(isq) == Lsize/2)  break;
            //weightI  += ave_u2[jtime](isq);
            //iweightI += i * ave_u2[jtime](isq);
            weight   += ave_u2[jtime](isq);
            iweight  += i * ave_u2[jtime](isq);
            i++;
        }
        fprintf(fp, "%d, %e, %e, %e, %e, %e \n", imax,  max, 
                     Lsize/2.0 - iweight/weight, avei,
                     sqrt(avei2-avei*avei), Lsize/2.0 - qcog[jtime]);
    }
}
